package impl

import (
	"context"
	"github.com/zeebe-io/zeebe/clients/go/entities"
	"github.com/zeebe-io/zeebe/clients/go/zbc"
	"gitlab.okta-solutions.com/mashroom/backend/cardpayment/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/bpms"
	"gitlab.okta-solutions.com/mashroom/backend/extras/api"
)

func ServeBpms(server *serverImpl) zbc.ZBClient {
	zbclient, err := bpms.ServeBpms("payment-service", bpms.MapHandler(map[string]bpms.Registrator{
		"get": func() (request interface{}, processor bpms.Processor) {
			req := &cardpayment.GetPropertyRequest{}
			return req, func(ctx context.Context, job entities.Job) (reply interface{}, enhance map[string]interface{}, err error) {
				resp, err := server.GetProperty(context.Background(), req)
				if err != nil {
					return nil, nil, err
				}
				paid := false
				for _, r := range resp.Payments {
					if r.Status == cardpayment.PaymentStatus_SUCCEEDED {
						for _, s := range r.Items {
							if s.GetTypeValue() == extras.ServiceType_LISTING_FEE {
								paid = true
								break
							}
						}
						if paid {
							break
						}
					}
				}
				return resp, map[string]interface{}{
					"paid": paid,
				}, err
			}
		},
	}))
	if err != nil {
		panic(err)
	}
	return zbclient
}
