package impl

import (
	"encoding/json"
	"github.com/stripe/stripe-go"
	"github.com/stripe/stripe-go/paymentintent"
	_ "github.com/stripe/stripe-go/webhookendpoint"
	"gitlab.okta-solutions.com/mashroom/backend/cardpayment/api"
	"gitlab.okta-solutions.com/mashroom/backend/common/generic/env"
	"math"
	"strings"
)

var (
	StripePublicKey  = env.ResolveString("STRIPE_PUBLIC_KEY", "pk_test_5PwKiXEb4Nv8nO1n3gSFz9P4")
	StripeSecretKey  = env.ResolveString("STRIPE_SECRET_KEY", "sk_test_fwK7y4sYjzWGwuzNEg3aDv2y")
	StripeWebhookURL = env.ResolveString("STRIPE_WEBHOOK_URL", "https://27f33d3d.ngrok.io/webhooks")
)

func init() {
	stripe.Key = StripeSecretKey
	stripe.NewIdempotencyKey()

	//TO DO - REGISTER WEBHOOK ENDPOINT IN STRIPE ACCOUNT
	//err := createWebhookEndpoint()
	//if err != nil {
	//	log.Errorln("create Stripe webhook endpoint error", err)
	//}
}

type PaymentIntentResponse struct {
	ID           string
	ClientSecret string
}

type WebhookResponse struct {
	ID     string
	Status cardpayment.PaymentStatus
}

type SEPAPaymentResponse struct {
	Status       bool
	ErrorMessage string
	ChargeID     string
}

func GetClientSecretImpl(amount int64, currency string) (*PaymentIntentResponse, error) {
	stripe.Key = StripeSecretKey

	params := &stripe.PaymentIntentParams{
		Amount:   stripe.Int64(amount),
		Currency: stripe.String(currency),
		PaymentMethodTypes: []*string{
			stripe.String("card"),
		},
		//SavePaymentMethod: stripe.Bool(true),
	}
	p, err := paymentintent.New(params)
	if err != nil {
		return nil, err
	}

	result := &PaymentIntentResponse{
		ID:           p.ID,
		ClientSecret: p.ClientSecret,
	}
	return result, nil
}

func HandleCallbackImpl(bytes []byte) (*WebhookResponse, error) {
	var stripeCallback StripeCallback
	err := json.Unmarshal(bytes, &stripeCallback)
	if err != nil {
		return nil, err
	}

	var result WebhookResponse

	if strings.HasPrefix(stripeCallback.Type, "charge") {
		var ch StripeCharge
		err = json.Unmarshal(bytes, &ch)
		if err != nil {
			return nil, err
		}

		result.ID = ch.Data.Object.ID
		result.Status = getStatus(ch.Data.Object.Status)
	} else if strings.HasPrefix(stripeCallback.Type, "source") {
		var src StripeSource
		err = json.Unmarshal(bytes, &src)
		if err != nil {
			return nil, err
		}

		result.ID = src.Data.Object.ID
		result.Status = getStatus(src.Data.Object.Status)
	} else if strings.HasPrefix(stripeCallback.Type, "payment_intent") {
		var pi PaymentIntent
		err = json.Unmarshal(bytes, &pi)
		if err != nil {
			return nil, err
		}

		result.ID = pi.Data.Object.ID
		result.Status = getStatus(pi.Data.Object.Status)
	}

	return &result, nil
}

func GetPublicKeyImpl() string {
	return StripePublicKey
}

func getStatus(customStatus string) cardpayment.PaymentStatus {
	if strings.Contains(customStatus, "succeeded") {
		return cardpayment.PaymentStatus_SUCCEEDED
	}
	if strings.Contains(customStatus, "failed") {
		return cardpayment.PaymentStatus_FAILED
	}

	return cardpayment.PaymentStatus_UNKNOWN_PAYMENT_STATUS
}

func Round(x float64, prec int) float64 {
	var rounder float64
	pow := math.Pow(10, float64(prec))
	intermed := x * pow
	_, frac := math.Modf(intermed)
	if frac >= 0.5 {
		rounder = math.Ceil(intermed)
	} else {
		rounder = math.Floor(intermed)
	}

	return rounder / pow
}

//TO DO - FUNC REGISTER WEBHOOK ENDPOINT IN STRIPE ACCOUNT
//func createWebhookEndpoint() error {
//	stripe.Key = StripeSecretKey
//
//	params := &stripe.WebhookEndpointListParams{}
//	params.Filters.AddFilter("limit", "", "10")
//	responseList := webhookendpoint.List(params)
//
//	var webHooks []*stripe.WebhookEndpoint
//	for responseList.Next() {
//		webHooks = append(webHooks, responseList.WebhookEndpoint())
//	}
//
//	if webHooks != nil && len(webHooks) > 0 {
//		for _, v := range webHooks {
//			if strings.EqualFold(v.URL, StripeWebhookURL) {
//				return nil
//			}
//		}
//	}
//
//	paramsNew := &stripe.WebhookEndpointParams{
//		URL: stripe.String(StripeWebhookURL),
//		EnabledEvents: []*string{
//			stripe.String("*"),
//		},
//	}
//	_, err := webhookendpoint.New(paramsNew)
//	if err != nil {
//		return err
//	}
//
//	return nil
//}
