package impl

import (
	"github.com/golang/protobuf/ptypes/timestamp"
	extras "gitlab.okta-solutions.com/mashroom/backend/extras/api"
)

type StripeCallback struct {
	Created         int    `json:"created"`
	Livemode        bool   `json:"livemode"`
	ID              string `json:"id"`
	Type            string `json:"type"`
	Object          string `json:"object"`
	PendingWebhooks int    `json:"pending_webhooks"`
	APIVersion      string `json:"api_version"`
}

type StripeCharge struct {
	Created         int         `json:"created"`
	Livemode        bool        `json:"livemode"`
	ID              string      `json:"id"`
	Type            string      `json:"type"`
	Object          string      `json:"object"`
	Request         interface{} `json:"request"`
	PendingWebhooks int         `json:"pending_webhooks"`
	APIVersion      string      `json:"api_version"`
	Data            struct {
		Object struct {
			ID                   string  `json:"id"`
			Object               string  `json:"object"`
			Amount               int     `json:"amount"`
			AmountRefunded       int     `json:"amount_refunded"`
			Application          *string `json:"application"`
			ApplicationFee       *string `json:"application_fee"`
			ApplicationFeeAmount *int64  `json:"application_fee_amount"`
			BalanceTransaction   string  `json:"balance_transaction"`
			Captured             bool    `json:"captured"`
			Created              int     `json:"created"`
			Currency             string  `json:"currency"`
			Customer             *string `json:"customer"`
			Description          string  `json:"description"`
			Destination          *string `json:"destination"`
			Dispute              *string `json:"dispute"`
			FailureCode          *string `json:"failure_code"`
			FailureMessage       *string `json:"failure_message"`
			FraudDetails         struct {
			} `json:"fraud_details"`
			Invoice  *string `json:"invoice"`
			Livemode bool    `json:"livemode"`
			Metadata struct {
			} `json:"metadata"`
			OnBehalfOf    *string     `json:"on_behalf_of"`
			Order         *string     `json:"order"`
			Outcome       interface{} `json:"outcome"`
			Paid          bool        `json:"paid"`
			PaymentIntent *string     `json:"payment_intent"`
			ReceiptEmail  *string     `json:"receipt_email"`
			ReceiptNumber *string     `json:"receipt_number"`
			Refunded      bool        `json:"refunded"`
			Refunds       struct {
				Object     string        `json:"object"`
				Data       []interface{} `json:"data"`
				HasMore    bool          `json:"has_more"`
				TotalCount int           `json:"total_count"`
				URL        string        `json:"url"`
			} `json:"refunds"`
			Review   *string     `json:"review"`
			Shipping interface{} `json:"shipping"`
			Source   struct {
				ID                string  `json:"id"`
				Object            string  `json:"object"`
				AddressCity       *string `json:"address_city"`
				AddressCountry    *string `json:"address_country"`
				AddressLine1      *string `json:"address_line1"`
				AddressLine1Check *string `json:"address_line1_check"`
				AddressLine2      *string `json:"address_line2"`
				AddressState      *string `json:"address_state"`
				AddressZip        string  `json:"address_zip"`
				AddressZipCheck   string  `json:"address_zip_check"`
				Brand             string  `json:"brand"`
				Country           string  `json:"country"`
				Customer          *string `json:"customer"`
				CvcCheck          string  `json:"cvc_check"`
				DynamicLast4      *string `json:"dynamic_last4"`
				ExpMonth          int     `json:"exp_month"`
				ExpYear           int     `json:"exp_year"`
				Fingerprint       string  `json:"fingerprint"`
				Funding           string  `json:"funding"`
				Last4             string  `json:"last4"`
				Metadata          struct {
				} `json:"metadata"`
				Name               string  `json:"name"`
				TokenizationMethod *string `json:"tokenization_method"`
			} `json:"source"`
			SourceTransfer      *string     `json:"source_transfer"`
			StatementDescriptor *string     `json:"statement_descriptor"`
			Status              string      `json:"status"`
			TransferData        interface{} `json:"transfer_data"`
			TransferGroup       *string     `json:"transfer_group"`
		} `json:"object"`
	} `json:"data"`
}

type StripeSource struct {
	Created         int         `json:"created"`
	Livemode        bool        `json:"livemode"`
	ID              string      `json:"id"`
	Type            string      `json:"type"`
	Object          string      `json:"object"`
	Request         interface{} `json:"request"`
	PendingWebhooks int         `json:"pending_webhooks"`
	APIVersion      string      `json:"api_version"`
	Data            struct {
		Object struct {
			ID                string `json:"id"`
			Object            string `json:"object"`
			AchCreditTransfer struct {
				AccountNumber string `json:"account_number"`
				RoutingNumber string `json:"routing_number"`
				Fingerprint   string `json:"fingerprint"`
				BankName      string `json:"bank_name"`
				SwiftCode     string `json:"swift_code"`
			} `json:"ach_credit_transfer"`
			Amount       interface{} `json:"amount"`
			ClientSecret string      `json:"client_secret"`
			Created      int         `json:"created"`
			Currency     string      `json:"currency"`
			Flow         string      `json:"flow"`
			Livemode     bool        `json:"livemode"`
			Metadata     struct {
			} `json:"metadata"`
			Owner struct {
				Address         interface{} `json:"address"`
				Email           string      `json:"email"`
				Name            interface{} `json:"name"`
				Phone           interface{} `json:"phone"`
				VerifiedAddress interface{} `json:"verified_address"`
				VerifiedEmail   interface{} `json:"verified_email"`
				VerifiedName    interface{} `json:"verified_name"`
				VerifiedPhone   interface{} `json:"verified_phone"`
			} `json:"owner"`
			Receiver struct {
				Address                string `json:"address"`
				AmountCharged          int    `json:"amount_charged"`
				AmountReceived         int    `json:"amount_received"`
				AmountReturned         int    `json:"amount_returned"`
				RefundAttributesMethod string `json:"refund_attributes_method"`
				RefundAttributesStatus string `json:"refund_attributes_status"`
			} `json:"receiver"`
			StatementDescriptor interface{} `json:"statement_descriptor"`
			Status              string      `json:"status"`
			Type                string      `json:"type"`
			Usage               string      `json:"usage"`
		} `json:"object"`
	} `json:"data"`
}

type StripeSourceTransaction struct {
	Created         int         `json:"created"`
	Livemode        bool        `json:"livemode"`
	ID              string      `json:"id"`
	Type            string      `json:"type"`
	Object          string      `json:"object"`
	Request         interface{} `json:"request"`
	PendingWebhooks int         `json:"pending_webhooks"`
	APIVersion      string      `json:"api_version"`
	Data            struct {
		Object struct {
			ID            string `json:"id"`
			Object        string `json:"object"`
			Amount        int    `json:"amount"`
			BitcoinAmount int    `json:"bitcoin_amount"`
			Created       int    `json:"created"`
			Currency      string `json:"currency"`
			Receiver      string `json:"receiver"`
		} `json:"object"`
	} `json:"data"`
}

type PaymentIntent struct {
	Created         int         `json:"created"`
	Livemode        bool        `json:"livemode"`
	ID              string      `json:"id"`
	Type            string      `json:"type"`
	Object          string      `json:"object"`
	Request         interface{} `json:"request"`
	PendingWebhooks int         `json:"pending_webhooks"`
	APIVersion      string      `json:"api_version"`
	Data            struct {
		Object struct {
			ID                   string      `json:"id"`
			Object               string      `json:"object"`
			AllowedSourceTypes   []string    `json:"allowed_source_types"`
			Amount               int         `json:"amount"`
			AmountCapturable     int         `json:"amount_capturable"`
			AmountReceived       int         `json:"amount_received"`
			Application          interface{} `json:"application"`
			ApplicationFeeAmount interface{} `json:"application_fee_amount"`
			CanceledAt           interface{} `json:"canceled_at"`
			CancellationReason   interface{} `json:"cancellation_reason"`
			CaptureMethod        string      `json:"capture_method"`
			Charges              struct {
				Object     string        `json:"object"`
				Data       []interface{} `json:"data"`
				HasMore    bool          `json:"has_more"`
				TotalCount int           `json:"total_count"`
				URL        string        `json:"url"`
			} `json:"charges"`
			ClientSecret       interface{} `json:"client_secret"`
			ConfirmationMethod string      `json:"confirmation_method"`
			Created            int         `json:"created"`
			Currency           string      `json:"currency"`
			Customer           interface{} `json:"customer"`
			Description        string      `json:"description"`
			LastPaymentError   interface{} `json:"last_payment_error"`
			Livemode           bool        `json:"livemode"`
			Metadata           struct {
				OrderID string `json:"order_id"`
			} `json:"metadata"`
			NextSourceAction interface{} `json:"next_source_action"`
			OnBehalfOf       interface{} `json:"on_behalf_of"`
			ReceiptEmail     string      `json:"receipt_email"`
			Review           interface{} `json:"review"`
			Shipping         struct {
				Address struct {
					City       string      `json:"city"`
					Country    string      `json:"country"`
					Line1      string      `json:"line1"`
					Line2      interface{} `json:"line2"`
					PostalCode string      `json:"postal_code"`
					State      interface{} `json:"state"`
				} `json:"address"`
				Carrier        interface{} `json:"carrier"`
				Name           string      `json:"name"`
				Phone          interface{} `json:"phone"`
				TrackingNumber interface{} `json:"tracking_number"`
			} `json:"shipping"`
			Source              string      `json:"source"`
			StatementDescriptor string      `json:"statement_descriptor"`
			Status              string      `json:"status"`
			TransferData        interface{} `json:"transfer_data"`
			TransferGroup       interface{} `json:"transfer_group"`
		} `json:"object"`
	} `json:"data"`
}

type PaymentForInvoice struct {
	Services      []*extras.Service    `json:"services"`
	CreatedAt     *timestamp.Timestamp `json:"created_at"`
	InvoiceNumber uint64               `json:"invoice_number"`
}
